import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

public class CSVOperatorClass extends Object {
    private static final int BUF_SIZE = 102400;

    private static final String CSV_ENCODING = "utf-8";

    private static boolean StopOperation;

    private static String EscapeQuotes(String str) {
        String result = new String("");

        for (int i = 0; i < str.length(); i++) {
            char ch = str.charAt(i);

            if (ch == '"') {
                result = result + '"';
            }
            result = result + ch;
        }

        return result;
    }

    private static Vector ParseCSVLine(String line, char delimiter) {
        boolean in_cell = true, cell_quoted = false;
        String  cell = new String("");
        Vector  result = new Vector();

        for (int i = 0; i < line.length(); i++) {
            if (line.charAt(i) == '"') {
                if (in_cell) {
                    if (cell_quoted) {
                        if (i + 1 < line.length()) {
                            if (line.charAt(i + 1) == '"') {
                                cell = cell + line.charAt(i++);
                            } else {
                                result.addElement(cell);
                                cell = new String("");

                                in_cell     = false;
                                cell_quoted = false;
                            }
                        } else {
                            result.addElement(cell);
                            cell = new String("");

                            in_cell     = false;
                            cell_quoted = false;
                        }
                    } else if (cell.length() == 0) {
                        cell_quoted = true;
                    } else {
                        cell = cell + line.charAt(i);
                    }
                }
            } else if (line.charAt(i) == delimiter) {
                if (in_cell) {
                    if (cell_quoted) {
                        cell = cell + line.charAt(i);
                    } else {
                        result.addElement(cell);
                        cell = new String("");
                    }
                } else {
                    in_cell = true;
                }
            } else {
                cell = cell + line.charAt(i);
            }
        }
        if (in_cell) {
            result.addElement(cell);
        }

        return result;
    }

    public static synchronized void StartOperation() {
        StopOperation = false;
    }

    public static synchronized void StopOperation() {
        StopOperation = true;
    }

    public static synchronized boolean OperationStopped() {
        return StopOperation;
    }

    public static int ExportToCSV(String root_name, String file_name, int wallet_id, Calendar month, char delimiter,
                                  Alert progress_alert) throws Exception {
        int            exported_records = 0;
        Vector         transactions;
        Exception      exception = null;
        FileConnection file = null;
        OutputStream   stream = null;
        Gauge          progress;

        try {
            transactions = TransactionStorageClass.GetListFilteredByWalletIdMonthSortedByDate(wallet_id, month);

            progress = new Gauge(null, false, transactions.size() + 1, 0);

            progress_alert.setIndicator(progress);

            String csv = new String("");

            for (int i = 0; i < transactions.size() && !OperationStopped(); i++) {
                TransactionClass transaction = (TransactionClass)transactions.elementAt(i);

                WalletClass   wallet   = WalletStorageClass.Get(transaction.GetWalletId());
                CategoryClass category = CategoryStorageClass.Get(transaction.GetCategoryId());

                if (wallet != null && category != null) {
                    String date          = EscapeQuotes(UtilClass.FormatDate(transaction.GetDate()));
                    String wallet_name   = EscapeQuotes(wallet.GetName());
                    String category_name = EscapeQuotes(category.GetName());
                    String amount        = EscapeQuotes(UtilClass.MoneyToString(transaction.GetAmount()));
                    String comment       = EscapeQuotes(transaction.GetComment());

                    csv = csv + "\"" + date          + "\"" + delimiter +
                                "\"" + wallet_name   + "\"" + delimiter +
                                "\"" + category_name + "\"" + delimiter +
                                "\"" + amount        + "\"" + delimiter +
                                "\"" + comment       + "\"\r\n";

                    exported_records++;
                }

                progress.setValue(i);
            }

            if (file_name.toLowerCase().endsWith(".csv")) {
                    file = (FileConnection)Connector.open("file:///" + root_name + file_name,          Connector.READ_WRITE);
            } else {
                    file = (FileConnection)Connector.open("file:///" + root_name + file_name + ".csv", Connector.READ_WRITE);
            }

            if (file.exists()) {
                file.truncate(0);
            } else {
                file.create();
            }

            stream = file.openDataOutputStream();
            stream.write(csv.getBytes(CSV_ENCODING));
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (file != null) {
                try {
                    file.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return exported_records;
        }
    }

    public static int[] ImportFromCSV(String root_name, String file_name, char delimiter,
                                      Alert progress_alert) throws Exception {
        int               imported_records = 0, import_errors = 0, chars_read;
        StringBuffer      buffer;
        Exception         exception = null;
        FileConnection    file = null;
        InputStream       stream = null;
        InputStreamReader stream_reader = null;
        Gauge             progress;

        try {
            if (file_name.toLowerCase().endsWith(".csv")) {
                    file = (FileConnection)Connector.open("file:///" + root_name + file_name,          Connector.READ);
            } else {
                    file = (FileConnection)Connector.open("file:///" + root_name + file_name + ".csv", Connector.READ);
            }

            stream        = file.openDataInputStream();
            stream_reader = new InputStreamReader(stream, CSV_ENCODING);
            buffer        = new StringBuffer();

            char buf[] = new char[BUF_SIZE];

            while ((chars_read = stream_reader.read(buf, 0, BUF_SIZE)) != -1) {
                buffer.append(buf, 0, chars_read);
            }

            Vector csv = UtilClass.SplitToLines(buffer.toString());

            progress = new Gauge(null, false, csv.size() + 1, 0);

            progress_alert.setIndicator(progress);

            for (int i = 0; i < csv.size() && !OperationStopped(); i++) {
                Vector csv_row = ParseCSVLine((String)csv.elementAt(i), delimiter);

                if (csv_row.size() == 4 || csv_row.size() == 5) {
                    try {
                        Date   date          = UtilClass.ParseDate((String)csv_row.elementAt(0));
                        String wallet_name   = (String)csv_row.elementAt(1);
                        String category_name = (String)csv_row.elementAt(2);
                        long   amount        = UtilClass.StringToMoney((String)csv_row.elementAt(3));
                        String comment;

                        if (csv_row.size() == 5) {
                            comment = (String)csv_row.elementAt(4);
                        } else {
                            comment = "";
                        }

                        int           wallet_id, category_id;
                        WalletClass   wallet;
                        CategoryClass category;

                        wallet = WalletStorageClass.SearchByName(wallet_name);

                        if (wallet == null) {
                            WalletStorageClass.Add(new WalletClass(wallet_name));

                            wallet = WalletStorageClass.SearchByName(wallet_name);

                            if (wallet == null) {
                                throw (new Exception("Error adding wallet"));
                            } else {
                                wallet_id = wallet.GetId();
                            }
                        } else {
                            wallet_id = wallet.GetId();
                        }

                        category = CategoryStorageClass.SearchByName(category_name);

                        if (category == null) {
                            CategoryStorageClass.Add(new CategoryClass(category_name));

                            category = CategoryStorageClass.SearchByName(category_name);

                            if (category == null) {
                                throw (new Exception("Error adding category"));
                            } else {
                                category_id = category.GetId();
                            }
                        } else {
                            category_id = category.GetId();
                        }

                        TransactionStorageClass.Add(new TransactionClass(wallet_id, category_id, amount, date, comment));

                        imported_records++;
                    } catch (Exception ex) {
                        import_errors++;
                    }
                } else {
                    import_errors++;
                }

                progress.setValue(i);
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (stream_reader != null) {
                try {
                    stream_reader.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (file != null) {
                try {
                    file.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            int result[] = new int[2];

            result[0] = imported_records;
            result[1] = import_errors;

            return result;
        }
    }
}
