import java.util.*;
import javax.microedition.rms.*;

public class CategoryStorageClass extends Object {
    private static final String STORAGE_NAME = "CATEGORY_STORAGE";

    private static String ValidateErrorMsg;

    private static Hashtable Hash = new Hashtable();

    private static void SortByName(Vector vector, int left, int right) {
        if (right > left) {
            int i = left - 1;
            int j = right;

            while (true) {
                while (((CategoryClass)vector.elementAt(++i)).GetName().compareTo(((CategoryClass)vector.elementAt(right)).GetName()) < 0);
                while (j > 0) {
                    if (((CategoryClass)vector.elementAt(--j)).GetName().compareTo(((CategoryClass)vector.elementAt(right)).GetName()) <= 0) {
                        break;
                    }
                }
                if (i >= j) {
                    break;
                }

                CategoryClass item = (CategoryClass)vector.elementAt(i);

                vector.setElementAt(vector.elementAt(j), i);
                vector.setElementAt(item, j);
            }

            CategoryClass item = (CategoryClass)vector.elementAt(i);

            vector.setElementAt(vector.elementAt(right), i);
            vector.setElementAt(item, right);

            SortByName(vector, left,  i - 1);
            SortByName(vector, i + 1, right);
        }
    }

    public static void Initialize() throws Exception {
        int         id;
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage                    = RecordStore.openRecordStore(STORAGE_NAME, true);
            RecordEnumeration rec_enum = storage.enumerateRecords(null, null, false);

            while(rec_enum.hasNextElement()) {
                id = rec_enum.nextRecordId();

                Hash.put(new Integer(id), new CategoryClass(id, storage.getRecord(id)));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static boolean Validate(CategoryClass category) {
        if (category.GetName().equals("")) {
            ValidateErrorMsg = "Category name is empty";

            return false;
        } else {
            Enumeration hash_enum = Hash.elements();

            while (hash_enum.hasMoreElements()) {
                if (category.GetName().equals(((CategoryClass)hash_enum.nextElement()).GetName())) {
                    ValidateErrorMsg = "Duplicate category name";

                    return false;
                }
            }

            return true;
        }
    }

    public static String GetValidateErrorMsg() {
        return ValidateErrorMsg;
    }

    public static CategoryClass Get(int id) {
        return (CategoryClass)Hash.get(new Integer(id));
    }

    public static void Add(CategoryClass category) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, true);

            byte byte_array[] = category.ToByteArray();

            int id = storage.addRecord(byte_array, 0, byte_array.length);

            category.SetId(id);

            Hash.put(new Integer(id), category);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Delete(int id) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);
            storage.deleteRecord(id);

            Hash.remove(new Integer(id));
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static void Replace(int id, CategoryClass category) throws Exception {
        Exception   exception = null;
        RecordStore storage = null;

        try {
            storage = RecordStore.openRecordStore(STORAGE_NAME, false);

            byte byte_array[] = category.ToByteArray();

            storage.setRecord(id, byte_array, 0, byte_array.length);

            category.SetId(id);

            Hash.put(new Integer(id), category);
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (storage != null) {
                try {
                    storage.closeRecordStore();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        }
    }

    public static int GetCount() {
        return Hash.size();
    }

    public static CategoryClass SearchByName(String name) {
        Enumeration hash_enum = Hash.elements();

        while (hash_enum.hasMoreElements()) {
            CategoryClass category = (CategoryClass)hash_enum.nextElement();

            if (name.equals(category.GetName())) {
                return category;
            }
        }

        return null;
    }

    public static Vector GetListSortedByName() {
        Vector      result = new Vector();
        Enumeration hash_enum = Hash.elements();

        while (hash_enum.hasMoreElements()) {
            result.addElement(hash_enum.nextElement());
        }

        SortByName(result, 0, result.size() - 1);

        return result;
    }
}
