import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;

public class GDocsOperatorClass extends Object {
    private static final int BUF_SIZE = 102400;

    private static final String CLIENT_LOGIN_URL             = "https://www.google.com/accounts/ClientLogin";
    private static final String CLIENT_LOGIN_ACCOUNT_TYPE    = "HOSTED_OR_GOOGLE";
    private static final String CLIENT_LOGIN_SERVICE_DOCLIST = "writely";
    private static final String CLIENT_LOGIN_SERVICE_SSHEETS = "wise";
    private static final String CLIENT_LOGIN_SOURCE          = "LightWallet";
    private static final String GDOCS_URL                    = "http://docs.google.com/feeds/default/private/full";
    private static final String GDOCS_SEARCH_URL             = "http://docs.google.com/feeds/default/private/full/-/spreadsheet";
    private static final String GDOCS_SEARCH_TITLE_EXACT     = "true";
    private static final String GDOCS_EXPORT_URL             = "http://spreadsheets.google.com/feeds/download/spreadsheets/Export";
    private static final String GDOCS_EXPORT_FORMAT          = "csv";
    private static final String GDATA_VERSION                = "3.0";
    private static final String USER_AGENT                   = "LightWallet";
    private static final String CSV_ENCODING                 = "utf-8";

    private static boolean StopOperation;

    private static String EscapeQuotes(String str) {
        String result = new String("");

        for (int i = 0; i < str.length(); i++) {
            char ch = str.charAt(i);

            if (ch == '"') {
                result = result + '"';
            }
            result = result + ch;
        }

        return result;
    }

    private static Vector ParseCSVLine(String line, char delimiter) {
        boolean in_cell = true, cell_quoted = false;
        String  cell = new String("");
        Vector  result = new Vector();

        for (int i = 0; i < line.length(); i++) {
            if (line.charAt(i) == '"') {
                if (in_cell) {
                    if (cell_quoted) {
                        if (i + 1 < line.length()) {
                            if (line.charAt(i + 1) == '"') {
                                cell = cell + line.charAt(i++);
                            } else {
                                result.addElement(cell);
                                cell = new String("");

                                in_cell     = false;
                                cell_quoted = false;
                            }
                        } else {
                            result.addElement(cell);
                            cell = new String("");

                            in_cell     = false;
                            cell_quoted = false;
                        }
                    } else if (cell.length() == 0) {
                        cell_quoted = true;
                    } else {
                        cell = cell + line.charAt(i);
                    }
                }
            } else if (line.charAt(i) == delimiter) {
                if (in_cell) {
                    if (cell_quoted) {
                        cell = cell + line.charAt(i);
                    } else {
                        result.addElement(cell);
                        cell = new String("");
                    }
                } else {
                    in_cell = true;
                }
            } else {
                cell = cell + line.charAt(i);
            }
        }
        if (in_cell) {
            result.addElement(cell);
        }

        return result;
    }

    private static String GetClientLoginToken(String acct_name, String acct_pass, String service) throws Exception {
        int                   chars_read;
        StringBuffer          buffer;
        String                result = null;
        Exception             exception = null;
        HttpConnection        connection = null;
        OutputStream          output_stream = null;
        InputStream           input_stream = null;
        InputStreamReader     stream_reader = null;

        try {
            connection = (HttpConnection)Connector.open(CLIENT_LOGIN_URL);

            connection.setRequestMethod(HttpConnection.POST);
            connection.setRequestProperty("User-Agent", USER_AGENT);
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

            output_stream  = connection.openDataOutputStream();
            String request = "accountType=" + CLIENT_LOGIN_ACCOUNT_TYPE +
                             "&Email="      + UtilClass.URLEncode(acct_name) +
                             "&Passwd="     + UtilClass.URLEncode(acct_pass) +
                             "&service="    + UtilClass.URLEncode(service) +
                             "&source="     + CLIENT_LOGIN_SOURCE;

            output_stream.write(request.getBytes());

            input_stream  = connection.openDataInputStream();
            stream_reader = new InputStreamReader(input_stream);
            buffer        = new StringBuffer();

            char buf[] = new char[BUF_SIZE];

            while ((chars_read = stream_reader.read(buf, 0, BUF_SIZE)) != -1) {
                buffer.append(buf, 0, chars_read);
            }

            if (connection.getResponseCode() == HttpConnection.HTTP_OK) {
                String tmpstr = buffer.toString();
                int    begin  = tmpstr.indexOf("Auth=");

                if (begin != -1) {
                    int end = tmpstr.indexOf("\r\n", begin + 5);

                    if (end == -1) {
                        end = tmpstr.indexOf("\n", begin + 5);
                    }
                    if (end == -1) {
                        end = tmpstr.length();
                    }

                    result = tmpstr.substring(begin + 5, end);
                } else {
                    result = "";
                }
            } else {
                String error;
                String tmpstr = buffer.toString();
                int    begin  = tmpstr.indexOf("Error=");

                if (begin != -1) {
                    int end = tmpstr.indexOf("\r\n", begin + 6);

                    if (end == -1) {
                        end = tmpstr.indexOf("\n", begin + 6);
                    }
                    if (end == -1) {
                        end = tmpstr.length();
                    }

                    error = tmpstr.substring(begin + 6, end);
                } else {
                    error = "Unknown ClientLogin error";
                }

                throw (new Exception(error));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (stream_reader != null) {
                try {
                    stream_reader.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (input_stream != null) {
                try {
                    input_stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (output_stream != null) {
                try {
                    output_stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return result;
        }
    }

    private static String GetSpreadsheetResourceId(String token, String sheet_name) throws Exception {
        int                   chars_read;
        StringBuffer          buffer;
        String                result = null;
        Exception             exception = null;
        HttpConnection        connection = null;
        InputStream           stream = null;
        InputStreamReader     stream_reader = null;

        try {
            connection = (HttpConnection)Connector.open(GDOCS_SEARCH_URL +
                                                        "?title="        + UtilClass.URLEncode(sheet_name) +
                                                        "&title-exact="  + GDOCS_SEARCH_TITLE_EXACT);

            connection.setRequestMethod(HttpConnection.GET);
            connection.setRequestProperty("GData-Version", GDATA_VERSION);
            connection.setRequestProperty("Authorization", "GoogleLogin auth=" + token);
            connection.setRequestProperty("User-Agent", USER_AGENT);

            stream        = connection.openDataInputStream();
            stream_reader = new InputStreamReader(stream);
            buffer        = new StringBuffer();

            char buf[] = new char[BUF_SIZE];

            while ((chars_read = stream_reader.read(buf, 0, BUF_SIZE)) != -1) {
                buffer.append(buf, 0, chars_read);
            }

            if (connection.getResponseCode() == HttpConnection.HTTP_OK) {
                String tmpstr = buffer.toString();
                int    begin  = tmpstr.indexOf("<gd:resourceId>spreadsheet:");

                if (begin != -1) {
                    int end = tmpstr.indexOf("</gd:resourceId>", begin + 27);

                    if (end != -1) {
                        result = tmpstr.substring(begin + 27, end);
                    } else {
                        result = "";
                    }
                } else {
                    result = "";
                }
            } else {
                throw (new Exception(String.valueOf(connection.getResponseCode()) + " " +
                                                    connection.getResponseMessage()));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (stream_reader != null) {
                try {
                    stream_reader.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return result;
        }
    }

    public static synchronized void StartOperation() {
        StopOperation = false;
    }

    public static synchronized void StopOperation() {
        StopOperation = true;
    }

    public static synchronized boolean OperationStopped() {
        return StopOperation;
    }

    public static int ExportToGDocs(String acct_name, String acct_pass, String sheet_name, int wallet_id, Calendar month,
                                    Alert progress_alert) throws Exception {
        int            exported_records = 0;
        Vector         transactions;
        Exception      exception = null;
        HttpConnection connection = null;
        OutputStream   output_stream = null;
        InputStream    input_stream = null;
        Gauge          progress;

        try {
            transactions = TransactionStorageClass.GetListFilteredByWalletIdMonthSortedByDate(wallet_id, month);

            progress = new Gauge(null, false, transactions.size() + 1, 0);

            progress_alert.setIndicator(progress);

            String csv = new String("");

            for (int i = 0; i < transactions.size() && !OperationStopped(); i++) {
                TransactionClass transaction = (TransactionClass)transactions.elementAt(i);

                WalletClass   wallet   = WalletStorageClass.Get(transaction.GetWalletId());
                CategoryClass category = CategoryStorageClass.Get(transaction.GetCategoryId());

                if (wallet != null && category != null) {
                    String date          = EscapeQuotes(UtilClass.FormatDate(transaction.GetDate()));
                    String wallet_name   = EscapeQuotes(wallet.GetName());
                    String category_name = EscapeQuotes(category.GetName());
                    String amount        = EscapeQuotes(UtilClass.MoneyToString(transaction.GetAmount()));
                    String comment       = EscapeQuotes(transaction.GetComment());

                    csv = csv + "\"" + date          + "\"," +
                                "\"" + wallet_name   + "\"," +
                                "\"" + category_name + "\"," +
                                "\"" + amount        + "\"," +
                                "\"" + comment       + "\"\r\n";

                    exported_records++;
                }

                progress.setValue(i);
            }

            String token = GetClientLoginToken(acct_name, acct_pass, CLIENT_LOGIN_SERVICE_DOCLIST);

            connection = (HttpConnection)Connector.open(GDOCS_URL);

            connection.setRequestMethod(HttpConnection.POST);
            connection.setRequestProperty("GData-Version", GDATA_VERSION);
            connection.setRequestProperty("Authorization", "GoogleLogin auth=" + token);
            connection.setRequestProperty("Slug", UtilClass.URLEncode(sheet_name));
            connection.setRequestProperty("User-Agent", USER_AGENT);
            connection.setRequestProperty("Content-Type", "text/csv");

            output_stream = connection.openDataOutputStream();

            output_stream.write(csv.getBytes(CSV_ENCODING));

            input_stream = connection.openDataInputStream();

            if (connection.getResponseCode() != HttpConnection.HTTP_CREATED) {
                throw (new Exception(String.valueOf(connection.getResponseCode()) + " " +
                                                    connection.getResponseMessage()));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (input_stream != null) {
                try {
                    input_stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (output_stream != null) {
                try {
                    output_stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            return exported_records;
        }
    }

    public static int[] ImportFromGDocs(String acct_name, String acct_pass, String sheet_name,
                                        Alert progress_alert) throws Exception {
        int               imported_records = 0, import_errors = 0, chars_read;
        StringBuffer      buffer;
        Exception         exception = null;
        HttpConnection    connection = null;
        InputStream       stream = null;
        InputStreamReader stream_reader = null;
        Gauge             progress;

        try {
            String doclist_token = GetClientLoginToken(acct_name, acct_pass, CLIENT_LOGIN_SERVICE_DOCLIST);
            String ssheets_token = GetClientLoginToken(acct_name, acct_pass, CLIENT_LOGIN_SERVICE_SSHEETS);
            String resource_id   = GetSpreadsheetResourceId(doclist_token, sheet_name);

            if (!resource_id.equals("")) {
                connection = (HttpConnection)Connector.open(GDOCS_EXPORT_URL +
                                                            "?key="          + UtilClass.URLEncode(resource_id) +
                                                            "&exportFormat=" + GDOCS_EXPORT_FORMAT);

                connection.setRequestMethod(HttpConnection.GET);
                connection.setRequestProperty("GData-Version", GDATA_VERSION);
                connection.setRequestProperty("Authorization", "GoogleLogin auth=" + ssheets_token);
                connection.setRequestProperty("User-Agent", USER_AGENT);

                stream        = connection.openDataInputStream();
                stream_reader = new InputStreamReader(stream, CSV_ENCODING);
                buffer        = new StringBuffer();

                char buf[] = new char[BUF_SIZE];

                while ((chars_read = stream_reader.read(buf, 0, BUF_SIZE)) != -1) {
                    buffer.append(buf, 0, chars_read);
                }

                if (connection.getResponseCode() == HttpConnection.HTTP_OK) {
                    Vector csv = UtilClass.SplitToLines(buffer.toString());

                    progress = new Gauge(null, false, csv.size() + 1, 0);

                    progress_alert.setIndicator(progress);

                    for (int i = 0; i < csv.size() && !OperationStopped(); i++) {
                        Vector csv_row = ParseCSVLine((String)csv.elementAt(i), ',');

                        if (csv_row.size() == 4 || csv_row.size() == 5) {
                            try {
                                Date   date          = UtilClass.ParseDate((String)csv_row.elementAt(0));
                                String wallet_name   = (String)csv_row.elementAt(1);
                                String category_name = (String)csv_row.elementAt(2);
                                long   amount        = UtilClass.StringToMoney((String)csv_row.elementAt(3));
                                String comment;

                                if (csv_row.size() == 5) {
                                    comment = (String)csv_row.elementAt(4);
                                } else {
                                    comment = "";
                                }

                                int           wallet_id, category_id;
                                WalletClass   wallet;
                                CategoryClass category;

                                wallet = WalletStorageClass.SearchByName(wallet_name);

                                if (wallet == null) {
                                    WalletStorageClass.Add(new WalletClass(wallet_name));

                                    wallet = WalletStorageClass.SearchByName(wallet_name);

                                    if (wallet == null) {
                                        throw (new Exception("Error adding wallet"));
                                    } else {
                                        wallet_id = wallet.GetId();
                                    }
                                } else {
                                    wallet_id = wallet.GetId();
                                }

                                category = CategoryStorageClass.SearchByName(category_name);

                                if (category == null) {
                                    CategoryStorageClass.Add(new CategoryClass(category_name));

                                    category = CategoryStorageClass.SearchByName(category_name);

                                    if (category == null) {
                                        throw (new Exception("Error adding category"));
                                    } else {
                                        category_id = category.GetId();
                                    }
                                } else {
                                    category_id = category.GetId();
                                }

                                TransactionStorageClass.Add(new TransactionClass(wallet_id, category_id, amount, date, comment));

                                imported_records++;
                            } catch (Exception ex) {
                                import_errors++;
                            }
                        } else {
                            import_errors++;
                        }

                        progress.setValue(i);
                    }
                } else {
                    throw (new Exception(String.valueOf(connection.getResponseCode()) + " " +
                                                        connection.getResponseMessage()));
                }
            } else {
                throw (new Exception("Spreadsheet does not exist"));
            }
        } catch (Exception ex) {
            exception = ex;
        } finally {
            if (stream_reader != null) {
                try {
                    stream_reader.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (Exception ex) {
                    // Ignore
                }
            }
        }
        if (exception != null) {
            throw (exception);
        } else {
            int result[] = new int[2];

            result[0] = imported_records;
            result[1] = import_errors;

            return result;
        }
    }
}
