import java.util.*;

public class WalletTransactionsTableClass extends CalendarTableClass {
    private static final String TITLE[]            = {"Date", "", "Amount"};
    private static final String TITLE_CATEGORY     = "Category";
    private static final String TITLE_COMMENT      = "Comment";
    private static final int    ALIGN[]            = {CalendarTableClass.ITEM_ALIGN_LEFT,
                                                      CalendarTableClass.ITEM_ALIGN_LEFT,
                                                      CalendarTableClass.ITEM_ALIGN_RIGHT};
    private static final int    CHANGEABLE_COL_NUM = 1;
    private static final int    FLOAT_COL_NUM      = 1;

    public static final int     VIEW_CATEGORIES    = 1;
    public static final int     VIEW_COMMENTS      = 2;

    private int      WalletId, CategoryFilterId, ViewType, TransactionIds[];
    private String   Categories[], Comments[];
    private Calendar DateFilter;

    public WalletTransactionsTableClass(Calendar month, int wallet_id, int category_filter_id, Calendar date_filter,
                                        int view_type, String wallet_name) {
        super(wallet_name, "", month, TITLE, ALIGN, FLOAT_COL_NUM);

        WalletId         = wallet_id;
        CategoryFilterId = category_filter_id;
        ViewType         = view_type;

        if (date_filter != null) {
            DateFilter = Calendar.getInstance(date_filter.getTimeZone());
            DateFilter.setTime(date_filter.getTime());
        } else {
            DateFilter = null;
        }

        if (ViewType == VIEW_CATEGORIES) {
            SetColumnTitle(CHANGEABLE_COL_NUM, TITLE_CATEGORY);
        } else if (ViewType == VIEW_COMMENTS) {
            SetColumnTitle(CHANGEABLE_COL_NUM, TITLE_COMMENT);
        } else {
            SetColumnTitle(CHANGEABLE_COL_NUM, "");
        }

        RefreshData();
    }

    protected void RefreshData() {
        Calendar transaction_date = Calendar.getInstance();
        Vector   transactions;

        try {
            if (CategoryFilterId == 0 && DateFilter == null) {
                SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)) + " : " +
                         UtilClass.MoneyToString(TransactionStorageClass.GetBalanceForWalletIdMonthFilteredByCategoryIdDate(WalletId, CategoryFilterId, DateFilter, Month)));
            } else {
                SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)) + " (filtered) : " +
                         UtilClass.MoneyToString(TransactionStorageClass.GetBalanceForWalletIdMonthFilteredByCategoryIdDate(WalletId, CategoryFilterId, DateFilter, Month)));
            }
        } catch (Exception ex) {
            if (CategoryFilterId == 0 && DateFilter == null) {
                SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)));
            } else {
                SetLabel(UtilClass.GetCalendarMonthName(Month) + " " + String.valueOf(Month.get(Calendar.YEAR)) +
                         " (filtered)");
            }
        }

        try {
            transactions = TransactionStorageClass.GetListForWalletIdMonthFilteredByCategoryIdDateSortedByDateRev(WalletId, CategoryFilterId, DateFilter, Month);

            TransactionIds = new int[transactions.size()];
            Data           = new String[transactions.size()][GetCols()];
            Categories     = new String[transactions.size()];
            Comments       = new String[transactions.size()];
            Rows           = Data.length;
            SelectedRow    = 0;

            for (int i = 0; i < transactions.size(); i++) {
                TransactionClass transaction = (TransactionClass)transactions.elementAt(i);
                CategoryClass    category    = CategoryStorageClass.Get(transaction.GetCategoryId());

                if (category != null) {
                    String category_name = category.GetName();

                    transaction_date.setTime(transaction.GetDate());

                    TransactionIds[i] = transaction.GetId();
                    Data[i][0]        = UtilClass.ZeroPad(transaction_date.get(Calendar.DAY_OF_MONTH)) + " " +
                                        UtilClass.ZeroPad(transaction_date.get(Calendar.HOUR_OF_DAY)) + ":" +
                                        UtilClass.ZeroPad(transaction_date.get(Calendar.MINUTE));

                    if (ViewType == VIEW_CATEGORIES) {
                        Data[i][1] = category_name;
                    } else if (ViewType == VIEW_COMMENTS) {
                        Data[i][1] = transaction.GetComment();
                    } else {
                        Data[i][1] = "";
                    }

                    if (transaction.GetAmount() > 0) {
                        Data[i][2] = "+" + UtilClass.MoneyToString(transaction.GetAmount());
                    } else {
                        Data[i][2] =       UtilClass.MoneyToString(transaction.GetAmount());
                    }

                    Categories[i] = category_name;
                    Comments[i]   = transaction.GetComment();
                } else {
                    throw (new Exception("Invalid category"));
                }
            }
        } catch (Exception ex) {
            TransactionIds = new int[0];
            Data           = new String[0][GetCols()];
            Rows           = Data.length;
            SelectedRow    = 0;
        }
    }

    protected void keyPressed(int key_code) {
        if (key_code == KEY_STAR) {
            SwitchView();
        }

        super.keyPressed(key_code);
    }

    public int GetTransactionId(int selected_index) {
        return TransactionIds[selected_index];
    }

    public int GetViewType() {
        return ViewType;
    }

    public void SwitchView() {
        if (ViewType == VIEW_CATEGORIES) {
            for (int i = 0; i < Rows; i++) {
                Data[i][1] = Comments[i];
            }

            ViewType = VIEW_COMMENTS;

            SetColumnTitle(CHANGEABLE_COL_NUM, TITLE_COMMENT);
        } else if (ViewType == VIEW_COMMENTS) {
            for (int i = 0; i < Rows; i++) {
                Data[i][1] = Categories[i];
            }

            ViewType = VIEW_CATEGORIES;

            SetColumnTitle(CHANGEABLE_COL_NUM, TITLE_CATEGORY);
        }

        repaint();
    }
}
